@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* struct module -- pack values into and (out of) strings */
 
@@ -60,9 +30,7 @@ Whitespace between formats is ignored.\n\
 The variable struct.error is an exception raised on errors.";
 
 #include "Python.h"
-#include "mymath.h"
 
-#include <limits.h>
 #include <ctype.h>
 
 
@@ -97,6 +65,8 @@ typedef struct { char c; void *x; } s_void_p;
 #define DOUBLE_ALIGN (sizeof(s_double) - sizeof(double))
 #define VOID_P_ALIGN (sizeof(s_void_p) - sizeof(void *))
 
+#define STRINGIFY(x)    #x
+
 #ifdef __powerc
 #pragma options align=reset
 #endif
@@ -105,9 +75,7 @@ typedef struct { char c; void *x; } s_void_p;
    if it isn't one */
 
 static int
-get_long(v, p)
-	PyObject *v;
-	long *p;
+get_long(PyObject *v, long *p)
 {
 	long x = PyInt_AsLong(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -124,9 +92,7 @@ get_long(v, p)
 /* Same, but handling unsigned long */
 
 static int
-get_ulong(v, p)
-	PyObject *v;
-	unsigned long *p;
+get_ulong(PyObject *v, unsigned long *p)
 {
 	if (PyLong_Check(v)) {
 		unsigned long x = PyLong_AsUnsignedLong(v);
@@ -150,10 +116,9 @@ get_ulong(v, p)
 /* XXX Inf/NaN are not handled quite right (but underflow is!) */
 
 static int
-pack_float(x, p, incr)
-	double x; /* The number to pack */
-	char *p;  /* Where to pack the high order byte */
-	int incr; /* 1 for big-endian; -1 for little-endian */
+pack_float(double x, /* The number to pack */
+           char *p,  /* Where to pack the high order byte */
+           int incr) /* 1 for big-endian; -1 for little-endian */
 {
 	int s;
 	int e;
@@ -222,10 +187,9 @@ pack_float(x, p, incr)
 }
 
 static int
-pack_double(x, p, incr)
-	double x; /* The number to pack */
-	char *p;  /* Where to pack the high order byte */
-	int incr; /* 1 for big-endian; -1 for little-endian */
+pack_double(double x, /* The number to pack */
+            char *p,  /* Where to pack the high order byte */
+            int incr) /* 1 for big-endian; -1 for little-endian */
 {
 	int s;
 	int e;
@@ -315,9 +279,8 @@ pack_double(x, p, incr)
 }
 
 static PyObject *
-unpack_float(p, incr)
-	char *p;  /* Where the high order byte is */
-	int incr; /* 1 for big-endian; -1 for little-endian */
+unpack_float(const char *p,  /* Where the high order byte is */
+             int incr)       /* 1 for big-endian; -1 for little-endian */
 {
 	int s;
 	int e;
@@ -359,9 +322,8 @@ unpack_float(p, incr)
 }
 
 static PyObject *
-unpack_double(p, incr)
-	char *p;  /* Where the high order byte is */
-	int incr; /* 1 for big-endian; -1 for little-endian */
+unpack_double(const char *p,  /* Where the high order byte is */
+              int incr)       /* 1 for big-endian; -1 for little-endian */
 {
 	int s;
 	int e;
@@ -427,90 +389,69 @@ typedef struct _formatdef {
 	char format;
 	int size;
 	int alignment;
-	PyObject* (*unpack) Py_PROTO((const char *,
-				      const struct _formatdef *));
-	int (*pack) Py_PROTO((char *,
-			      PyObject *,
-			      const struct _formatdef *));
+	PyObject* (*unpack)(const char *,
+			    const struct _formatdef *);
+	int (*pack)(char *, PyObject *,
+		    const struct _formatdef *);
 } formatdef;
 
 static PyObject *
-nu_char(p, f)
-	const char *p;
-	const formatdef *f;
+nu_char(const char *p, const formatdef *f)
 {
 	return PyString_FromStringAndSize(p, 1);
 }
 
 static PyObject *
-nu_byte(p, f)
-	const char *p;
-	const formatdef *f;
+nu_byte(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong((long) *(signed char *)p);
 }
 
 static PyObject *
-nu_ubyte(p, f)
-	const char *p;
-	const formatdef *f;
+nu_ubyte(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong((long) *(unsigned char *)p);
 }
 
 static PyObject *
-nu_short(p, f)
-	const char *p;
-	const formatdef *f;
+nu_short(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong((long) *(short *)p);
 }
 
 static PyObject *
-nu_ushort(p, f)
-	const char *p;
-	const formatdef *f;
+nu_ushort(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong((long) *(unsigned short *)p);
 }
 
 static PyObject *
-nu_int(p, f)
-	const char *p;
-	const formatdef *f;
+nu_int(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong((long) *(int *)p);
 }
 
 static PyObject *
-nu_uint(p, f)
-	const char *p;
-	const formatdef *f;
+nu_uint(const char *p, const formatdef *f)
 {
 	unsigned int x = *(unsigned int *)p;
 	return PyLong_FromUnsignedLong((unsigned long)x);
 }
 
 static PyObject *
-nu_long(p, f)
-	const char *p;
-	const formatdef *f;
+nu_long(const char *p, const formatdef *f)
 {
 	return PyInt_FromLong(*(long *)p);
 }
 
 static PyObject *
-nu_ulong(p, f)
-	const char *p;
-	const formatdef *f;
+nu_ulong(const char *p, const formatdef *f)
 {
 	return PyLong_FromUnsignedLong(*(unsigned long *)p);
 }
 
 static PyObject *
-nu_float(p, f)
-	const char *p;
-	const formatdef *f;
+nu_float(const char *p, const formatdef *f)
 {
 	float x;
 	memcpy((char *)&x, p, sizeof(float));
@@ -518,9 +459,7 @@ nu_float(p, f)
 }
 
 static PyObject *
-nu_double(p, f)
-	const char *p;
-	const formatdef *f;
+nu_double(const char *p, const formatdef *f)
 {
 	double x;
 	memcpy((char *)&x, p, sizeof(double));
@@ -528,31 +467,43 @@ nu_double(p, f)
 }
 
 static PyObject *
-nu_void_p(p, f)
-	const char *p;
-	const formatdef *f;
+nu_void_p(const char *p, const formatdef *f)
 {
 	return PyLong_FromVoidPtr(*(void **)p);
 }
 
 static int
-np_byte(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_byte(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	if (get_long(v, &x) < 0)
 		return -1;
+	if (x < -128 || x > 127){
+		PyErr_SetString(StructError,
+				"byte format requires -128<=number<=127");
+		return -1;
+	}
 	*p = (char)x;
 	return 0;
 }
 
 static int
-np_char(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_ubyte(char *p, PyObject *v, const formatdef *f)
+{
+	long x;
+	if (get_long(v, &x) < 0)
+		return -1;
+	if (x < 0 || x > 255){
+		PyErr_SetString(StructError,
+				"ubyte format requires 0<=number<=255");
+		return -1;
+	}
+	*p = (char)x;
+	return 0;
+}
+
+static int
+np_char(char *p, PyObject *v, const formatdef *f)
 {
 	if (!PyString_Check(v) || PyString_Size(v) != 1) {
 		PyErr_SetString(StructError,
@@ -564,23 +515,38 @@ np_char(p, v, f)
 }
 
 static int
-np_short(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_short(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	if (get_long(v, &x) < 0)
 		return -1;
+	if (x < SHRT_MIN || x > SHRT_MAX){
+		PyErr_SetString(StructError,
+				"short format requires " STRINGIFY(SHRT_MIN)
+                                "<=number<=" STRINGIFY(SHRT_MAX));
+		return -1;
+	}
 	* (short *)p = (short)x;
 	return 0;
 }
 
 static int
-np_int(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_ushort(char *p, PyObject *v, const formatdef *f)
+{
+	long x;
+	if (get_long(v, &x) < 0)
+		return -1;
+	if (x < 0 || x > USHRT_MAX){
+		PyErr_SetString(StructError,
+				"short format requires 0<=number<=" STRINGIFY(USHRT_MAX));
+		return -1;
+	}
+	* (unsigned short *)p = (unsigned short)x;
+	return 0;
+}
+
+static int
+np_int(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	if (get_long(v, &x) < 0)
@@ -590,10 +556,7 @@ np_int(p, v, f)
 }
 
 static int
-np_uint(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_uint(char *p, PyObject *v, const formatdef *f)
 {
 	unsigned long x;
 	if (get_ulong(v, &x) < 0)
@@ -603,10 +566,7 @@ np_uint(p, v, f)
 }
 
 static int
-np_long(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_long(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	if (get_long(v, &x) < 0)
@@ -616,10 +576,7 @@ np_long(p, v, f)
 }
 
 static int
-np_ulong(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_ulong(char *p, PyObject *v, const formatdef *f)
 {
 	unsigned long x;
 	if (get_ulong(v, &x) < 0)
@@ -629,10 +586,7 @@ np_ulong(p, v, f)
 }
 
 static int
-np_float(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_float(char *p, PyObject *v, const formatdef *f)
 {
 	float x = (float)PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -645,10 +599,7 @@ np_float(p, v, f)
 }
 
 static int
-np_double(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_double(char *p, PyObject *v, const formatdef *f)
 {
 	double x = PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -661,10 +612,7 @@ np_double(p, v, f)
 }
 
 static int
-np_void_p(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+np_void_p(char *p, PyObject *v, const formatdef *f)
 {
 	void *x = PyLong_AsVoidPtr(v);
 	if (x == NULL && PyErr_Occurred()) {
@@ -681,12 +629,12 @@ np_void_p(p, v, f)
 static formatdef native_table[] = {
 	{'x',	sizeof(char),	0,		NULL},
 	{'b',	sizeof(char),	0,		nu_byte,	np_byte},
-	{'B',	sizeof(char),	0,		nu_ubyte,	np_byte},
+	{'B',	sizeof(char),	0,		nu_ubyte,	np_ubyte},
 	{'c',	sizeof(char),	0,		nu_char,	np_char},
 	{'s',	sizeof(char),	0,		NULL},
 	{'p',	sizeof(char),	0,		NULL},
 	{'h',	sizeof(short),	SHORT_ALIGN,	nu_short,	np_short},
-	{'H',	sizeof(short),	SHORT_ALIGN,	nu_ushort,	np_short},
+	{'H',	sizeof(short),	SHORT_ALIGN,	nu_ushort,	np_ushort},
 	{'i',	sizeof(int),	INT_ALIGN,	nu_int,		np_int},
 	{'I',	sizeof(int),	INT_ALIGN,	nu_uint,	np_uint},
 	{'l',	sizeof(long),	LONG_ALIGN,	nu_long,	np_long},
@@ -698,9 +646,7 @@ static formatdef native_table[] = {
 };
 
 static PyObject *
-bu_int(p, f)
-	const char *p;
-	const formatdef *f;
+bu_int(const char *p, const formatdef *f)
 {
 	long x = 0;
 	int i = f->size;
@@ -716,9 +662,7 @@ bu_int(p, f)
 }
 
 static PyObject *
-bu_uint(p, f)
-	const char *p;
-	const formatdef *f;
+bu_uint(const char *p, const formatdef *f)
 {
 	unsigned long x = 0;
 	int i = f->size;
@@ -732,26 +676,19 @@ bu_uint(p, f)
 }
 
 static PyObject *
-bu_float(p, f)
-	const char *p;
-	const formatdef *f;
+bu_float(const char *p, const formatdef *f)
 {
 	return unpack_float(p, 1);
 }
 
 static PyObject *
-bu_double(p, f)
-	const char *p;
-	const formatdef *f;
+bu_double(const char *p, const formatdef *f)
 {
 	return unpack_double(p, 1);
 }
 
 static int
-bp_int(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+bp_int(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	int i;
@@ -766,10 +703,7 @@ bp_int(p, v, f)
 }
 
 static int
-bp_uint(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+bp_uint(char *p, PyObject *v, const formatdef *f)
 {
 	unsigned long x;
 	int i;
@@ -784,10 +718,7 @@ bp_uint(p, v, f)
 }
 
 static int
-bp_float(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+bp_float(char *p, PyObject *v, const formatdef *f)
 {
 	double x = PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -799,10 +730,7 @@ bp_float(p, v, f)
 }
 
 static int
-bp_double(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+bp_double(char *p, PyObject *v, const formatdef *f)
 {
 	double x = PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -832,9 +760,7 @@ static formatdef bigendian_table[] = {
 };
 
 static PyObject *
-lu_int(p, f)
-	const char *p;
-	const formatdef *f;
+lu_int(const char *p, const formatdef *f)
 {
 	long x = 0;
 	int i = f->size;
@@ -850,9 +776,7 @@ lu_int(p, f)
 }
 
 static PyObject *
-lu_uint(p, f)
-	const char *p;
-	const formatdef *f;
+lu_uint(const char *p, const formatdef *f)
 {
 	unsigned long x = 0;
 	int i = f->size;
@@ -866,26 +790,19 @@ lu_uint(p, f)
 }
 
 static PyObject *
-lu_float(p, f)
-	const char *p;
-	const formatdef *f;
+lu_float(const char *p, const formatdef *f)
 {
 	return unpack_float(p+3, -1);
 }
 
 static PyObject *
-lu_double(p, f)
-	const char *p;
-	const formatdef *f;
+lu_double(const char *p, const formatdef *f)
 {
 	return unpack_double(p+7, -1);
 }
 
 static int
-lp_int(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+lp_int(char *p, PyObject *v, const formatdef *f)
 {
 	long x;
 	int i;
@@ -900,10 +817,7 @@ lp_int(p, v, f)
 }
 
 static int
-lp_uint(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+lp_uint(char *p, PyObject *v, const formatdef *f)
 {
 	unsigned long x;
 	int i;
@@ -918,10 +832,7 @@ lp_uint(p, v, f)
 }
 
 static int
-lp_float(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+lp_float(char *p, PyObject *v, const formatdef *f)
 {
 	double x = PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -933,10 +844,7 @@ lp_float(p, v, f)
 }
 
 static int
-lp_double(p, v, f)
-	char *p;
-	PyObject *v;
-	const formatdef *f;
+lp_double(char *p, PyObject *v, const formatdef *f)
 {
 	double x = PyFloat_AsDouble(v);
 	if (x == -1 && PyErr_Occurred()) {
@@ -967,8 +875,7 @@ static formatdef lilendian_table[] = {
 
 
 static const formatdef *
-whichtable(pfmt)
-	const char **pfmt;
+whichtable(char **pfmt)
 {
 	const char *fmt = (*pfmt)++; /* May be backed out of later */
 	switch (*fmt) {
@@ -997,9 +904,7 @@ whichtable(pfmt)
 /* Get the table entry for a format code */
 
 static const formatdef *
-getentry(c, f)
-	int c;
-	const formatdef *f;
+getentry(int c, const formatdef *f)
 {
 	for (; f->format != '\0'; f++) {
 		if (f->format == c) {
@@ -1014,10 +919,7 @@ getentry(c, f)
 /* Align a size according to a format code */
 
 static int
-align(size, c, e)
-	int size;
-	int c;
-	const formatdef *e;
+align(int size, int c, const formatdef *e)
 {
 	if (e->format == c) {
 		if (e->alignment) {
@@ -1033,9 +935,7 @@ align(size, c, e)
 /* calculate the size of a format string */
 
 static int
-calcsize(fmt, f)
-	const char *fmt;
-	const formatdef *f;
+calcsize(const char *fmt, const formatdef *f)
 {
 	const formatdef *e;
 	const char *s;
@@ -1089,15 +989,13 @@ Return size of C struct described by format string fmt.\n\
 See struct.__doc__ for more on format strings.";
 
 static PyObject *
-struct_calcsize(self, args)
-	PyObject *self; /* Not used */
-	PyObject *args;
+struct_calcsize(PyObject *self, PyObject *args)
 {
 	char *fmt;
 	const formatdef *f;
 	int size;
 
-	if (!PyArg_ParseTuple(args, "s", &fmt))
+	if (!PyArg_ParseTuple(args, "s:calcsize", &fmt))
 		return NULL;
 	f = whichtable(&fmt);
 	size = calcsize(fmt, f);
@@ -1113,9 +1011,7 @@ Return string containing values v1, v2, ... packed according to fmt.\n\
 See struct.__doc__ for more on format strings.";
 
 static PyObject *
-struct_pack(self, args)
-	PyObject *self; /* Not used */
-	PyObject *args;
+struct_pack(PyObject *self, PyObject *args)
 {
 	const formatdef *f, *e;
 	PyObject *format, *result, *v;
@@ -1128,7 +1024,8 @@ struct_pack(self, args)
 	if (args == NULL || !PyTuple_Check(args) ||
 	    (n = PyTuple_Size(args)) < 1)
         {
-		PyErr_BadArgument();
+		PyErr_SetString(PyExc_TypeError, 
+			"struct.pack requires at least one argument");
 		return NULL;
 	}
 	format = PyTuple_GetItem(args, 0);
@@ -1252,9 +1149,7 @@ to fmt.  Requires len(string)==calcsize(fmt).\n\
 See struct.__doc__ for more on format strings.";
 
 static PyObject *
-struct_unpack(self, args)
-	PyObject *self; /* Not used */
-	PyObject *args;
+struct_unpack(PyObject *self, PyObject *args)
 {
 	const formatdef *f, *e;
 	char *str, *start, *fmt, *s;
@@ -1262,7 +1157,7 @@ struct_unpack(self, args)
 	int len, size, num;
 	PyObject *res, *v;
 
-	if (!PyArg_ParseTuple(args, "ss#", &fmt, &start, &len))
+	if (!PyArg_ParseTuple(args, "ss#:unpack", &fmt, &start, &len))
 		return NULL;
 	f = whichtable(&fmt);
 	size = calcsize(fmt, f);
@@ -1359,7 +1254,7 @@ static PyMethodDef struct_methods[] = {
 /* Module initialization */
 
 DL_EXPORT(void)
-initstruct()
+initstruct(void)
 {
 	PyObject *m, *d;
 
